/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <stdio.h>

#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#define POINTS 24

/*----------------------------------------------------------------
 *  databox markers
 *----------------------------------------------------------------*/

void setM (GtkDataboxMarker *marker, GtkWidget *box,
           GtkDataboxMarkerPosition position,
           GtkDataboxTextPosition label_position,
           gchar *label,
           gboolean boxed)
{
   marker->position = position;
   marker->label_position = label_position;
   marker->type = GTK_DATABOX_MARKER_TRIANGLE;
   marker->label = gtk_widget_create_pango_layout (box, label);
   marker->boxed = boxed;
}

void setL (GtkDataboxMarker *marker, GtkWidget *box,
           GtkDataboxMarkerPosition position,
           GtkDataboxTextPosition label_position,
           gchar *label,
           gboolean boxed)
{
   marker->position = position;
   marker->label_position = label_position;
   marker->type = GTK_DATABOX_MARKER_DASHED_LINE;
   marker->label = gtk_widget_create_pango_layout (box, label);
   marker->boxed = boxed;
}

void setT (GtkDataboxText *text, GtkWidget *box,
           GtkDataboxTextPosition position,
           gchar *label,
           gboolean boxed)
{
   text->position = position;
   text->text = gtk_widget_create_pango_layout (box, label);
   text->boxed = boxed;
}

static void
create_markers (void)
{
   GtkWidget *window = NULL;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *close_button;
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *separator;
   GtkDataboxMarker *markers, *M;
   GtkDataboxText *texts, *T;
   gint index;
   gfloat *X;
   gfloat *Y;
   GdkColor color;
   
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request (window, 600, 600);

   g_signal_connect (GTK_OBJECT (window), "destroy",
		     G_CALLBACK (gtk_main_quit), NULL);

   gtk_window_set_title (GTK_WINDOW (window), "GtkDatabox: Marker");
   gtk_container_set_border_width (GTK_CONTAINER (window), 0);

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), box1);

   label =
      gtk_label_new
      ("Some markers (green) are shown below.\nThe text corresponds to the position of the text, relative to the marker.\n\nThere are also some texts (yellow).\n\nData points are red\n");
   gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

   box = gtk_databox_new ();

   color.red = 16383;
   color.green = 16383;
   color.blue = 16383;
   gtk_databox_set_background_color (GTK_DATABOX (box), color);
   
   X = g_new0 (gfloat, POINTS + 5);
   Y = g_new0 (gfloat, POINTS + 5);
   markers = M = g_new0 (GtkDataboxMarker, POINTS);
   texts = T = g_new0 (GtkDataboxText, 5);

   /* SET 1 */
   X [0] = 0.; Y [0] = 0.; 
   setM (&M[0], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_W, "West", FALSE);
   X [1] = 1.; Y [1] = 0.; 
   setM (&M[1], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_N, "North", FALSE);
   X [2] = 2.; Y [2] = 0.; 
   setM (&M[2], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_S, "South", FALSE);
   X [3] = 3.; Y [3] = 0.; 
   setM (&M[3], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_E, "East", FALSE);
   
   /* SET 2 */
   X [4] = 0.; Y [4] = 1.; 
   setM (&M[4], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_W, "West", TRUE);
   X [5] = 1.; Y [5] = 1.; 
   setM (&M[5], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_N, "North", TRUE);
   X [6] = 2.; Y [6] = 1.; 
   setM (&M[6], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_S, "South", TRUE);
   X [7] = 3.; Y [7] = 1.; 
   setM (&M[7], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_E, "East", TRUE);

   /* SET 3 */
   X [8] = 0.; Y [8] = 2.; 
   setM (&M[8], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_W, "West", TRUE);
   X [9] = 1.; Y [9] = 2.; 
   setM (&M[9], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_N, "North", TRUE);
   X [10] = 2.; Y [10] = 2.; 
   setM (&M[10], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_S, "South", TRUE);
   X [11] = 3.; Y [11] = 2.; 
   setM (&M[11], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_E, "East", TRUE);

   /* SET 4 */
   X [12] = 0.; Y [12] = 3.; 
   setM (&M[12], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_S, "South", TRUE);
   X [13] = 1.; Y [13] = 3.; 
   setM (&M[13], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_W, "West", TRUE);
   X [14] = 2.; Y [14] = 3.; 
   setM (&M[14], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_E, "East", TRUE);
   X [15] = 3.; Y [15] = 3.; 
   setM (&M[15], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_N, "North", TRUE);

   /* SET 5 */
   X [16] = 0.; Y [16] = 4.; 
   setM (&M[16], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_SW, "SW", TRUE);
   X [17] = 1.; Y [17] = 4.; 
   setM (&M[17], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_NW, "NW", TRUE);
   X [18] = 2.; Y [18] = 4.; 
   setM (&M[18], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_SE, "SE", TRUE);
   X [19] = 3.; Y [19] = 4.; 
   setM (&M[19], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_NE, "NE", TRUE);

   /* SET 6 */
   X [20] = - 0.5; Y [20] = - 0.5; 
   setL (&M[20], box, GTK_DATABOX_MARKER_N, GTK_DATABOX_TEXT_W, "West", TRUE);
   X [21] = - 0.5; Y [21] = 4.5; 
   setL (&M[21], box, GTK_DATABOX_MARKER_E, GTK_DATABOX_TEXT_S, "South", TRUE);
   X [22] = 3.5; Y [22] = - 0.5; 
   setL (&M[22], box, GTK_DATABOX_MARKER_W, GTK_DATABOX_TEXT_N, "North", TRUE);
   X [23] = 3.5; Y [23] = 4.5; 
   setL (&M[23], box, GTK_DATABOX_MARKER_S, GTK_DATABOX_TEXT_E, "East", TRUE);

   
   /* Text Set */
   X [POINTS + 0] = - 1.; Y [POINTS + 0] = - 1.;
   setT (&T[0], box, GTK_DATABOX_TEXT_N, "North", FALSE);
   X [POINTS + 1] = - 1.; Y [POINTS + 1] = + 5.;
   setT (&T[1], box, GTK_DATABOX_TEXT_E, "East", FALSE);
   X [POINTS + 2] = + 4.; Y [POINTS + 2] = - 1.;
   setT (&T[2], box, GTK_DATABOX_TEXT_S, "South", FALSE);
   X [POINTS + 3] = + 4.; Y [POINTS + 3] = + 5.;
   setT (&T[3], box, GTK_DATABOX_TEXT_W, "West", FALSE);
   X [POINTS + 4] = + 1.5; Y [POINTS + 4] = + 2.;
   setT (&T[4], box, GTK_DATABOX_TEXT_CENTER, "Centered", TRUE);

   color.red = 65535;
   color.green = 0;
   color.blue = 0;

   index = gtk_databox_data_add (GTK_DATABOX (box), POINTS + 5,
                         X, Y, color, GTK_DATABOX_POINTS, 3);
   
   color.red = 0;
   color.green = 65535;
   color.blue = 0;

   index = gtk_databox_data_add (GTK_DATABOX (box), POINTS,
                         X, Y, color, GTK_DATABOX_MARKER, 15);
   
   gtk_databox_marker_set_markers (GTK_DATABOX (box), index, markers, POINTS);

   color.red = 65535;
   color.green = 65535;
   color.blue = 0;

   index = gtk_databox_data_add (GTK_DATABOX (box), 5,
                         X + POINTS, Y + POINTS, color, GTK_DATABOX_TEXT, 5);
   
   gtk_databox_text_set_texts (GTK_DATABOX (box), index, texts, 5);

  
   gtk_databox_rescale (GTK_DATABOX (box));
   gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   close_button = gtk_button_new_with_label ("close");
   g_signal_connect_swapped (GTK_OBJECT (close_button), "clicked",
			     G_CALLBACK (gtk_main_quit), GTK_OBJECT (box));
   gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (close_button);

   gtk_widget_show_all (window);

}

gint
main (gint argc, char *argv[])
{
   gtk_init (&argc, &argv);

   create_markers ();
   gtk_main ();

   return 0;
}
